#ifndef INFO_HEADER
#define INFO_HEADER

#include "Absyn.h"
using namespace std;

class VarInfo {
public:
	VarInfo(string n, int l)
		: name(n), lineLocation(l) {}
	VarInfo(PIdent* ident)
		: VarInfo(ident->string_, ident->integer_) {}
	string name;
	Type *type;
	int lineLocation;

	bool operator<(const VarInfo &other) const {
		return name != other.name ? name < other.name : type < other.type;
	}
};

class ClassInfo;

class FunctionInfo : public VarInfo {
public:
	Block * block;
	vector<VarInfo*> arguments;
	ClassInfo *klass;
	FunctionInfo(PIdent *ident, ClassInfo *klass = NULL) : VarInfo(ident), klass(klass) {};
	~FunctionInfo() {
		for (auto &a : arguments)
			delete a;
	}
};

class ClassInfo : public VarInfo
{
	ClassInfo * parent;
public:
	vector<FunctionInfo*> functions;
	vector<VarInfo*> variables;

	ClassInfo(PIdent *ident, ClassInfo *parent = NULL) : VarInfo(ident), parent(parent) {};
	ClassInfo * getParent() const { return parent; };
	FunctionInfo* getFunction(string name) const {
		for (auto i : functions)
			if (i->name == name)
				return i;
		return parent ? parent->getFunction(name) : NULL;
	}

	VarInfo* getVar(string name) const {
		for (auto i : variables)
			if (i->name == name)
				return i;
		return parent ? parent->getVar(name) : NULL;
	}

	~ClassInfo() {
		for (auto &f : functions)
			delete f;

		for (auto &v : variables)
			delete v;
	}

};

class Scope {
public:
	vector<ClassInfo*> classes;
	vector<FunctionInfo*> functions;
	ClassInfo *currentClass;


	Scope() : currentClass(NULL) {}
	~Scope() {
		for (auto &c : classes)
			delete c;

		for (auto &f : functions)
			delete f;
	}
	ClassInfo* getClass(string name) const {
		for (auto i : classes)
			if (i->name == name)
				return i;
		return NULL;
	}
	FunctionInfo* getFunction(string name, bool local = false) const {
		if (currentClass && !local)
			if (auto *f = currentClass->getFunction(name))
				return f;

		for (auto i : functions)
			if (i->name == name)
				return i;

		return NULL;
	}

};

#endif